@@ -1,4 +1,5 @@
 from dataclasses import asdict, dataclass
+from typing import Union
 
 
 @dataclass
@@ -24,14 +25,14 @@ class AppCredentialsKeys:
 
 @dataclass
 class AppCredentials:
-    gcm: AppCredentialsGcm
-    fcm: AppCredentialsFcm
-    keys: AppCredentialsKeys
+    gcm: Union[AppCredentialsGcm, dict]
+    fcm: Union[AppCredentialsFcm, dict]
+    keys: Union[AppCredentialsKeys, dict]
 
     def __post_init__(self):
-        self.gcm = AppCredentialsGcm(**self.gcm)  # type: ignore
-        self.fcm = AppCredentialsFcm(**self.fcm)  # type: ignore
-        self.keys = AppCredentialsKeys(**self.keys)  # type: ignore
+        self.gcm: AppCredentialsGcm = AppCredentialsGcm(**self.gcm)  # type: ignore
+        self.fcm: AppCredentialsFcm = AppCredentialsFcm(**self.fcm)  # type: ignore
+        self.keys: AppCredentialsKeys = AppCredentialsKeys(**self.keys)  # type: ignore
 
     def __dict__(self):
         return asdict(self)
